package service

import (
	"fmt"
	"github.com/pkg/errors"
	"leit.com/aps_engine/models"
	"time"
)

/**调度引擎**/
type SchedulerSrv struct{
	PlantNr				int
	StartDate 			time.Time
	EndDate  			time.Time
	WorkPlaceDict		map[int]*WorkPlaceSrv		// 所有的设备对象
	WorkPlaceGrpDict	map[int]*WorkPlaceGrpSrv
	ToolDict			map[int]*ToolSrv			// 所有的工具对象
	ToolGrpDict			map[int]*ToolGrpSrv
	ArticleDict			map[string]*ArticleSrv		// 所有的物料对象
	StatusResourceDict	map[int]*StatusResourceSrv	// 所有的状态类资源对象
	QtyResourceDict		map[int]*QtyResourceSrv		// 所有的数量类资源对象
	PersonDict			map[int]*PersonSrv			// 所有人员对象
	SkillDict			map[int]*SkillSrv			// 所有技能对象
	OrderDict 			map[string]*OrderSrv
	TmSrv 				TimeModelSrv				// 时间模型
	ProjDict 			map[string]models.MeProject		// 激活的项目字典
	SchedRule 			ScheduleRuleSrv				// 调度规则
	SchedEventQueue		*SchedQueue					// 调度事件队列
}

// 调度引擎初始化
func(se *SchedulerSrv)Init(plantnr int, stdate, eddate time.Time){
	se.PlantNr				= plantnr
	se.StartDate			= stdate
	se.EndDate				= eddate
	se.WorkPlaceDict		= make(map[int]*WorkPlaceSrv)
	se.WorkPlaceGrpDict		= make(map[int]*WorkPlaceGrpSrv)
	se.ToolDict				= make(map[int]*ToolSrv)
	se.ToolGrpDict			= make(map[int]*ToolGrpSrv)
	se.ArticleDict			= make(map[string]*ArticleSrv)
	se.StatusResourceDict	= make(map[int]*StatusResourceSrv)
	se.QtyResourceDict		= make(map[int]*QtyResourceSrv)
	se.PersonDict			= make(map[int]*PersonSrv)
	se.SkillDict			= make(map[int]*SkillSrv)
	se.OrderDict			= make(map[string]*OrderSrv)
	se.TmSrv				= TimeModelSrv{PlantNr: plantnr}
	se.SchedEventQueue		= NewSchedQueue()
}

// 加载时间模型主数据
func(se *SchedulerSrv)LoadTimeModelData(){
	// 加载指定日期范畴内的时间模型相关主数据
	se.TmSrv.LoadBasicData(se.StartDate, se.EndDate)
	// 生成日模型的时间线列表
	se.TmSrv.GenDayModelLineArray()
}

// 加载激活的项目
func(se *SchedulerSrv)LoadProjectData()(err error){
	var(
		i			int
		projtab		models.MeProject
		projtablst	[]models.MeProject
	)
	// 初始化
	se.ProjDict	= make(map[string]models.MeProject)
	// 读取激活的项目数据
	projtab = models.MeProject{Plantnr: se.PlantNr}
	if projtablst, err = projtab.SelectActiveProjects(); err != nil{
		err = errors.New(fmt.Sprintf("获取工厂%d的激活的项目失败%v !", se.PlantNr, err))
		return
	}
	// 加载项目
	for i = 0; i < len(projtablst); i++ {
		se.ProjDict[projtablst[i].Projectid] = projtablst[i]
	}
	return
}

// 启动调度引擎
func(se *SchedulerSrv)Run()(err error){
	var(
		set		SchedEvent
	)

	// 从队列循环获取任务执行，直至队列为空
	for se.SchedEventQueue.Length() > 0 {
		set = se.SchedEventQueue.Pop()

		switch set.GetEvent() {
		case EVENT_TASK_SETUPBEG:
		case EVENT_TASK_SETUPEND:
		case EVENT_TASK_PLANBEG:
			oppbet := set.(OperationEvent)
			fmt.Println(fmt.Sprintf("订单:%s 工序:%d 计划开始:%s",oppbet.OrderId,oppbet.OperationNr,oppbet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedTaskPlanBeg(oppbet)
			continue

		case EVENT_TASK_PLANEND:
			oppeet := set.(OperationEvent)
			fmt.Println(fmt.Sprintf("订单:%s 工序:%d 计划结束:%s",oppeet.OrderId,oppeet.OperationNr,oppeet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedTaskPlanEnd(oppeet)
			continue

		case EVENT_TASK_POSTBEG:
		case EVENT_TASK_POSTEND:
		case EVENT_WP_STATUS_AVAIL:
			wpet := set.(WorkPlaceEvent)
			fmt.Println(fmt.Sprintf("|| 工位:%d 可使用时间:%s",wpet.WorkPlaceNr, wpet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedWPAvail(wpet)
			continue

		case EVENT_WP_STATUS_UNAVAIL:
			wpet := set.(WorkPlaceEvent)
			fmt.Println(fmt.Sprintf("XX 工位:%d 不可使用时间:%s",wpet.WorkPlaceNr, wpet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedWPUnAvail(wpet)
			continue

		case EVENT_TOOL_STATUS_AVAIL:
			tet := set.(ToolEvent)
			fmt.Println(fmt.Sprintf("|| 工具:%d 可使用时间:%s",tet.ToolNr, tet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedToolAvail(tet)
			continue

		case EVENT_TOOL_STATUS_UNAVAIL:
			tet := set.(ToolEvent)
			fmt.Println(fmt.Sprintf("XX 工具:%d 不可使用时间:%s",tet.ToolNr, tet.TriggerTime.Format("2006-01-02 15:04:05")))
			se.SchedToolUnAvail(tet)
			continue

		case EVENT_PERSON_STATUS_AVAIL:
		case EVENT_PERSON_STATUS_UNAVAIL:
		case EVENT_ART_CONSUME:
		case EVENT_ART_PRODUCE:
		case EVENT_QTYRES_CONSUME:
		case EVENT_QTYRES_PRODUCE:
		case EVENT_STATUSRES_AVAIL:
		case EVENT_STATUSRES_UNAVAIL:
		}
	}

	return
}

// 对计划开始工序进行调度
func(se *SchedulerSrv)SchedTaskPlanBeg(opevent OperationEvent){
	var(
		order		*OrderSrv
		op 			*OperationSrv
		wp			*WorkPlaceSrv
		wpg			*WorkPlaceGrpSrv
		t 			*ToolSrv
		tg 			*ToolGrpSrv
		newevent	SchedEvent
		optionalwplst	[]OptionalWorkPlaceSrv
		optoollst		[]OptionalToolSrv
		ok			bool
	)
	// 获取工序订单对象
	if order, ok = se.OrderDict[opevent.OrderId]; !ok {
		fmt.Println(fmt.Sprintf("工序事件的订单: %s 不存在", opevent.OrderId))
		return
	}
	// 获取工序对象
	if op, ok = order.OperationDict[opevent.GetOpKey()]; !ok {
		fmt.Println(fmt.Sprintf("工序事件的工序: %d 不存在于订单: %s 中", opevent.GetOpKey(), opevent.OrderId))
		return
	}
	// 获取可用工位
	optionalwplst = se.GetAvailWorkPlaceList(op.WorkPlaceGrpNr)
	if len(optionalwplst) <= 0 {
		//fmt.Println("工位组:",op.WorkPlaceGrpNr," 无可用工位！")
		// 获取工位组对象
		if wpg, ok = se.WorkPlaceGrpDict[op.WorkPlaceGrpNr]; !ok{
			//fmt.Println("工位组:",op.WorkPlaceGrpNr," 不存在！")
			return
		}
		// 无可用工位，则将任务放到工位组等待队列
		wpg.WaitTaskArray = append(wpg.WaitTaskArray, *op)
		return
	}
	if se.SchedRule.ToolConstraintToggle && op.ToolGrpNr > 0{
		// 获取可用工位
		optoollst = se.GetAvailToolList(op.ToolGrpNr)
		if len(optoollst) <= 0 {
			//fmt.Println("工具组:",op.ToolGrpNr," 无可用工位！")
			// 获取工具组对象
			if tg, ok = se.ToolGrpDict[op.ToolGrpNr]; !ok{
				//fmt.Println("工具组:",op.ToolGrpNr," 不存在！")
				return
			}
			// 无可用工位，则将任务放到工位组等待队列
			tg.WaitTaskArray = append(tg.WaitTaskArray, *op)
			return
		}
	}
	// 有可用工位，取第一个工位进行调度
	op.PlanStartTime 	= opevent.TriggerTime
	op.PlanEndTime		= op.PlanStartTime.Add(op.PlanDuration)
	op.PlanWorkPlaceNr	= optionalwplst[0].WorkPlaceNr
	if se.SchedRule.ToolConstraintToggle && op.ToolGrpNr > 0 {
		op.PlanToolNr = optoollst[0].ToolNr
	}

	// 基于工序开始生成工序结束事件
	fmt.Println("    创建工序结束事件：",op.OrderId,op.OperationNr,op.PlanEndTime.Format("2006-01-02 15:04:05"))
	newevent = op.GetPlanEndEvent(op.PlanEndTime)
	se.SchedEventQueue.Insert(newevent)

	// 基于工序开始生成设备事件,占用工位资源
	if wp, ok = se.WorkPlaceDict[op.PlanWorkPlaceNr]; ok {
		newevent = wp.GetUnAvailEvent(opevent.TriggerTime)
		se.SchedEventQueue.Insert(newevent)
		wp.Status = RES_STATUS_UNAVAIL
		wp.SchedTaskArray = append(wp.SchedTaskArray, *op)
	}
	// 考虑有限工具能力
	if se.SchedRule.ToolConstraintToggle && op.ToolGrpNr > 0{
		if t, ok = se.ToolDict[op.PlanToolNr]; ok {
			newevent = t.GetUnAvailEvent(opevent.TriggerTime)
			se.SchedEventQueue.Insert(newevent)
			t.Status = RES_STATUS_UNAVAIL
			t.SchedTaskArray = append(t.SchedTaskArray, *op)
		}
	}
}

// 对计划结束工序进行调度
func(se *SchedulerSrv)SchedTaskPlanEnd(opevent OperationEvent){
	var(
		order,nextorder		*OrderSrv
		op,nextop 			*OperationSrv
		wp			*WorkPlaceSrv
		t 			*ToolSrv
		newevent	SchedEvent
		nextoprel	OperationRelSrv
		ok			bool
	)
	// 获取工序订单对象
	if order, ok = se.OrderDict[opevent.OrderId]; !ok {
		fmt.Println(fmt.Sprintf("工序事件的订单: %s 不存在", opevent.OrderId))
		return
	}
	// 获取工序对象
	if op, ok = order.OperationDict[opevent.GetOpKey()]; !ok {
		fmt.Println(fmt.Sprintf("工序事件的工序: %d 不存在于订单: %s 中", opevent.GetOpKey(), opevent.OrderId))
		return
	}
	// 获取占用工位并释放资源
	if wp, ok = se.WorkPlaceDict[op.PlanWorkPlaceNr]; ok {
		newevent = wp.GetAvailEvent(opevent.TriggerTime)
		se.SchedEventQueue.Insert(newevent)
		wp.Status = RES_STATUS_AVAIL
	}else{
		fmt.Println(fmt.Sprintf("订单:%s 工序:%d 计划工位:%s 不存在！",op.OrderId,op.OperationNr,op.PlanWorkPlaceNr))
		return
	}
	if se.SchedRule.ToolConstraintToggle && op.ToolGrpNr > 0{
		if t, ok = se.ToolDict[op.PlanToolNr]; ok{
			newevent = t.GetAvailEvent(opevent.TriggerTime)
			se.SchedEventQueue.Insert(newevent)
			t.Status = RES_STATUS_AVAIL
		}
	}
	// 激活后道工序
	for _, nextoprel = range op.NextOpDict{
		if nextorder, ok = se.OrderDict[nextoprel.OrderId]; ok{
			if nextop, ok = nextorder.OperationDict[nextoprel.GetOpKey()]; ok{
				newevent = nextop.GetPlanStartEvent(opevent.TriggerTime)
				se.SchedEventQueue.Insert(newevent)
			}
		}else{
			fmt.Println(fmt.Sprintf("订单:%s 不存在",nextoprel.OrderId))
			continue
		}
	}
}

// 对工位可用进行调度
func(se *SchedulerSrv)SchedWPAvail(wpevent WorkPlaceEvent){
	var (
		wp			*WorkPlaceSrv
		wpg 		*WorkPlaceGrpSrv
		op			OperationSrv
		newevent	SchedEvent
		ok			bool
	)

	// 获取工位对象
	if wp, ok = se.WorkPlaceDict[wpevent.WorkPlaceNr]; !ok {
		fmt.Println(fmt.Sprintf("SchedWPAvail: 工位:%d 不存在",wpevent.WorkPlaceNr))
		return
	}
	// 获取工位组对象
	if wpg, ok = se.WorkPlaceGrpDict[wp.WorkPlaceGrpNr]; !ok {
		fmt.Println(fmt.Sprintf("工位组:%d 不存在",wp.WorkPlaceGrpNr))
		return
	}
	// 更新工位状态
	wp.Status = RES_STATUS_AVAIL
	//se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	// 将等待队列中的工序任务全部推入调度队列
	for _, op = range wpg.WaitTaskArray {
		fmt.Println("************ 获取等待队列中的等待任务：",op.OrderId,op.OperationNr)
		newevent = op.GetPlanStartEvent(wpevent.TriggerTime)
		se.SchedEventQueue.Insert(newevent)
	}
	wpg.WaitTaskArray = []OperationSrv{}
	//se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	return
}

// 对工位不可用进行调度
func(se *SchedulerSrv)SchedWPUnAvail(wpevent WorkPlaceEvent){
	var (
		wp			*WorkPlaceSrv
		ok			bool
	)

	// 获取工位对象
	if wp, ok = se.WorkPlaceDict[wpevent.WorkPlaceNr]; !ok {
		fmt.Println(fmt.Sprintf("SchedWPUnAvail: 工位:%d 不存在",wpevent.WorkPlaceNr))
		return
	}
	// 更新工位状态
	wp.Status = RES_STATUS_UNAVAIL
	//se.WorkPlaceDict[wp.WorkPlaceNr] = wp

	return
}

// 对工具可用进行调度
func(se *SchedulerSrv)SchedToolAvail(tevent ToolEvent){
	var (
		t			*ToolSrv
		tg 			*ToolGrpSrv
		op			OperationSrv
		newevent	SchedEvent
		ok			bool
	)

	// 获取工具对象
	if t, ok = se.ToolDict[tevent.ToolNr]; !ok {
		fmt.Println(fmt.Sprintf("SchedToolAvail: 工具:%d 不存在",tevent.ToolNr))
		return
	}
	// 获取工具组对象
	if tg, ok = se.ToolGrpDict[t.ToolGrpNr]; !ok {
		fmt.Println(fmt.Sprintf("工具组:%d 不存在",t.ToolGrpNr))
		return
	}
	// 更新工具状态
	t.Status = RES_STATUS_AVAIL
	// 将等待队列中的工序任务全部推入调度队列
	for _, op = range tg.WaitTaskArray {
		fmt.Println("************ 获取等待队列中的等待任务：",op.OrderId,op.OperationNr)
		newevent = op.GetPlanStartEvent(tevent.TriggerTime)
		se.SchedEventQueue.Insert(newevent)
	}
	tg.WaitTaskArray = []OperationSrv{}

	return
}

// 对工具不可用进行调度
func(se *SchedulerSrv)SchedToolUnAvail(tevent ToolEvent){
	var (
		t			*ToolSrv
		ok			bool
	)

	// 获取工位对象
	if t, ok = se.ToolDict[tevent.ToolNr]; !ok {
		fmt.Println(fmt.Sprintf("SchedToolUnAvail: 工具:%d 不存在",tevent.ToolNr))
		return
	}
	// 更新工位状态
	t.Status = RES_STATUS_UNAVAIL

	return
}

// 获取当前可用工位
func(se *SchedulerSrv)GetAvailWorkPlaceList(wpgnr int)(optionalwplst []OptionalWorkPlaceSrv){
	var(
		wpnr	int
		wp		*WorkPlaceSrv
		wpg 	*WorkPlaceGrpSrv
		ok		bool
	)

	// 获取工位组对象
	if wpg, ok = se.WorkPlaceGrpDict[wpgnr]; !ok {
		fmt.Println(fmt.Sprintf("GetAvailWorkPlaceList: 工位组:%d 不存在",wpgnr))
		return
	}

	optionalwplst = []OptionalWorkPlaceSrv{}
	for wpnr, _ = range wpg.WorkPlaceDict{
		if wp, ok = se.WorkPlaceDict[wpnr]; !ok{
			fmt.Println(fmt.Sprintf("GetAvailWorkPlaceList: 工位:%d 不存在",wpnr))
			return
		}

		if wp.Status == RES_STATUS_AVAIL {
			optionalwp := OptionalWorkPlaceSrv{WorkPlaceNr: wp.WorkPlaceNr, Priority: 0}
			optionalwplst = append(optionalwplst, optionalwp)
		}
	}

	return
}

// 获取当前可用工具
func(se *SchedulerSrv)GetAvailToolList(tgnr int)(opttoollst []OptionalToolSrv){
	var(
		tnr		int
		t		*ToolSrv
		tg 		*ToolGrpSrv
		ok		bool
	)

	// 获取工位组对象
	if tg, ok = se.ToolGrpDict[tgnr]; !ok {
		fmt.Println(fmt.Sprintf("GetAvailToolList: 工具组:%d 不存在",tgnr))
		return
	}
	fmt.Println("获取工具组的工具.......",tg.ToolGrpNr)
	opttoollst = []OptionalToolSrv{}
	for tnr, _ = range tg.ToolIdxDict{
		if t, ok = se.ToolDict[tnr]; !ok{
			fmt.Println(fmt.Sprintf("GetAvailToolList: 工位:%d 不存在",tnr))
			return
		}
		fmt.Println("    工具：",t.ToolNr,"    状态：",t.Status)
		if t.Status == RES_STATUS_AVAIL {
			optionalt := OptionalToolSrv{ToolNr: t.ToolNr, Priority: 0}
			opttoollst = append(opttoollst, optionalt)
		}
	}

	return
}

// 初始化测试环境，创建一些测试数据
func(se *SchedulerSrv)InitTestEnv()(err error){
	var(
		wp		*WorkPlaceSrv
		wpg 	*WorkPlaceGrpSrv
		t		*ToolSrv
		tg		*ToolGrpSrv
		art		*ArticleSrv
		order	*OrderSrv
		prevop	*OperationSrv
		op		*OperationSrv
		opr 	OperationRelSrv
		st,et 	time.Time
		tls		TimeLineSrv
	)
	// 初始化工厂号及计划区间
	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	se.Init(100, st, et)

	// 创建工位组
	wpg	= &WorkPlaceGrpSrv{}
	wpg.Init(10)

	// 创建工位
	wp = &WorkPlaceSrv{}
	wp.Init(1,10)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*24)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*28)
	et = time.Now().Add(time.Hour*48)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = *wp

	wp = &WorkPlaceSrv{}
	wp.Init(2,10)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*16)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*24)
	et = time.Now().Add(time.Hour*46)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = *wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	wpg	= &WorkPlaceGrpSrv{}
	wpg.Init(20)

	wp = &WorkPlaceSrv{}
	wp.Init(3,20)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*20)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*22)
	et = time.Now().Add(time.Hour*44)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = *wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	wpg	= &WorkPlaceGrpSrv{}
	wpg.Init(30)

	wp = &WorkPlaceSrv{}
	wp.Init(4,30)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*15)
	wp.TimeCurve.Init(st,et)

	tls = TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*21)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*23)
	et = time.Now().Add(time.Hour*50)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = *wp

	wp = &WorkPlaceSrv{}
	wp.Init(5,30)

	st = time.Now()
	et = time.Now().Add(time.Hour*24*10)
	wp.TimeCurve.Init(st,et)

	tls = TimeLineSrv{}
	st = time.Now()
	et = time.Now().Add(time.Hour*17)
	tls.Init(st, et)
	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*25)
	et = time.Now().Add(time.Hour*52)
	tls.Init(st, et)

	wp.TimeCurve.TimeLineArray = append(wp.TimeCurve.TimeLineArray, tls)
	se.WorkPlaceDict[wp.WorkPlaceNr] = wp
	wpg.WorkPlaceDict[wp.WorkPlaceNr] = *wp
	se.WorkPlaceGrpDict[wpg.WorkPlaceGrpNr] = wpg

	// 创建工具
	t = &ToolSrv{}
	t.Init(1, 10)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*5)
	et = time.Now().Add(time.Hour*10)
	t.UnAvailTimeLineArray = append(t.UnAvailTimeLineArray, tls)
	se.ToolDict[t.ToolNr] = t

	// 创建工具组
	tg = &ToolGrpSrv{}
	tg.Init(10)
	tg.ToolIdxDict[t.ToolNr] = t.ToolNr
	se.ToolGrpDict[tg.ToolGrpNr] = tg

	// 创建工具
	t = &ToolSrv{}
	t.Init(2, 20)

	tls = TimeLineSrv{}
	st = time.Now().Add(time.Hour*6)
	et = time.Now().Add(time.Hour*12)
	t.UnAvailTimeLineArray = append(t.UnAvailTimeLineArray, tls)
	se.ToolDict[t.ToolNr] = t

	// 创建工具组
	tg = &ToolGrpSrv{}
	tg.Init(20)
	tg.ToolIdxDict[t.ToolNr] = t.ToolNr
	se.ToolGrpDict[tg.ToolGrpNr] = tg

	// 创建物料
	art = &ArticleSrv{}
	art.Init("art001",100)
	se.ArticleDict[art.ArtId] = art

	// 创建工单及工序
	order = &OrderSrv{OrderId: "100001", ProjectId: "G18"}
	order.OperationArray 	= []OperationSrv{}
	order.OperationDict		= make(map[string]*OperationSrv)

	op = &OperationSrv{OrderId: "100001",OperationNr: 10, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 10, ToolGrpNr: 10, PlanDuration: time.Hour * 4}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = OperationRelSrv{OrderId: "100001", OperationNr: 20, SplitNr: 0, Relation: OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &OperationSrv{OrderId: "100001",OperationNr: 20, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 20, PlanDuration: time.Hour * 6}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	opr = OperationRelSrv{OrderId: "100001", OperationNr: 10, SplitNr: 0, Relation: OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = OperationRelSrv{OrderId: "100001", OperationNr: 30, SplitNr: 0, Relation: OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &OperationSrv{OrderId: "100001",OperationNr: 30, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 30, PlanDuration: time.Hour * 8}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	opr = OperationRelSrv{OrderId: "100001", OperationNr: 20, SplitNr: 0, Relation: OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	se.OrderDict[order.OrderId] = order

	order = &OrderSrv{OrderId: "100002", ProjectId: "G18"}
	order.OperationArray 	= []OperationSrv{}
	order.OperationDict		= make(map[string]*OperationSrv)

	op = &OperationSrv{OrderId: "100002",OperationNr: 10, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 10, ToolGrpNr: 10, PlanDuration: time.Hour * 5}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = OperationRelSrv{OrderId: "100002", OperationNr: 20, SplitNr: 0, Relation: OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &OperationSrv{OrderId: "100002",OperationNr: 20, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 20, PlanDuration: time.Hour * 7}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	opr = OperationRelSrv{OrderId: "100002", OperationNr: 10, SplitNr: 0, Relation: OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op

	prevop = op

	opr = OperationRelSrv{OrderId: "100002", OperationNr: 30, SplitNr: 0, Relation: OP_LINK_END_START}
	prevop.NextOpDict[opr.GetFullKey()] = opr

	op = &OperationSrv{OrderId: "100002",OperationNr: 30, SplitNr: 0,PlanQty: 10, WorkPlaceGrpNr: 30, PlanDuration: time.Hour * 9}
	op.PrevOpDict 			= make(map[string]OperationRelSrv)
	opr = OperationRelSrv{OrderId: "100002", OperationNr: 20, SplitNr: 0, Relation: OP_LINK_END_START}
	op.PrevOpDict[opr.GetFullKey()] = opr

	op.NextOpDict			= make(map[string]OperationRelSrv)
	order.OperationArray 	= append(order.OperationArray, *op)
	order.OperationDict[op.GetKey()] = op
	se.OrderDict[order.OrderId] = order

	// 初始化调度队列
	se.SchedEventQueue = NewSchedQueue()

	// 遍历设备时间曲线，生成设备状态事件，推入队列
	for _, wp_p := range se.WorkPlaceDict{
		for _, tls := range wp_p.TimeCurve.TimeLineArray{
			evt1 := wp_p.GetAvailEvent(tls.StartTime)
			se.SchedEventQueue.Insert(evt1)
			evt2 := wp_p.GetUnAvailEvent(tls.EndTime)
			se.SchedEventQueue.Insert(evt2)
		}
	}

	// 载入订单首工序事件
	for _, ord_p := range se.OrderDict{
		for _, op_p := range ord_p.OperationDict {
			if len(op_p.PrevOpDict) <= 0 {
				evt := op_p.GetPlanStartEvent(time.Now())
				opet := evt.(OperationEvent)
				fmt.Println("压入订单工序事件：",opet.OrderId,opet.OperationNr, opet.TriggerTime)
				se.SchedEventQueue.Insert(evt)
			}
		}
	}

	return
}